// RasClient.cxx

// #include <unistd.h>
#include <ptlib.h>
#include <ptlib/sockets.h>
#include <ptclib/asner.h>
#include "h225.h" 
#include "h323.h" 
#include "h323pdu.h" 
#include "RasClient.h"
#include "h323pdu.h"

BOOL RasClient::SendGRQ()
{
	H225_RasMessage grq_ras, grq_rpl;
	grq_ras.SetTag(H225_RasMessage::e_gatekeeperRequest);
	H225_GatekeeperRequest & grq_obj = grq_ras;

	grq_obj.m_requestSeqNum.SetValue( seq_num++ );
	grq_obj.m_protocolIdentifier.SetValue( H225_ProtocolID); 

	grq_obj.m_rasAddress.SetTag(H225_TransportAddress::e_ipAddress);
	H225_TransportAddress_ipAddress & ipAddress = grq_obj.m_rasAddress;
	ipAddress.m_ip[0]  = my_addr.Byte1();
	ipAddress.m_ip[1]  = my_addr.Byte2();
	ipAddress.m_ip[2]  = my_addr.Byte3();
	ipAddress.m_ip[3]  = my_addr.Byte4();
	ipAddress.m_port  = gk_port;

	grq_obj.m_endpointType.IncludeOptionalField( grq_obj.m_endpointType.e_terminal );
	grq_obj.IncludeOptionalField( grq_obj.e_gatekeeperIdentifier );    
	grq_obj.m_gatekeeperIdentifier.SetValue( GetGKID() );

	grq_obj.IncludeOptionalField( grq_obj.e_endpointAlias );    
	grq_obj.m_endpointAlias.SetSize(1);
	H323SetAliasAddress("gk_test", grq_obj.m_endpointAlias[0]);	

	// Send GRQ
	PBYTEArray rdbuf(2048), wtbuf(2048);
	PPER_Stream rdstrm(rdbuf), wtstrm(wtbuf);

	grq_ras.Encode ( wtstrm );
	wtstrm.CompleteEncoding();
	if(IsVerbose()) 
		cout << grq_ras << endl;

	PUDPSocket sock( gk_port );
	if( !sock.Connect(gk_addr) )
	{
		cout << "Can not connect to the Gatekeeper." << endl;
		sock.Close();  
		return FALSE;
	}

	cout << GetHostName() << " Sent to " << gk_addr << ":" << gk_port << endl;
	sock.Write(wtstrm.GetPointer(), wtstrm.GetSize());

	sock.SetReadTimeout(GetReadTimeout());
	if (!sock.ReadFrom( rdstrm.GetPointer(), rdstrm.GetSize(), gk_addr, gk_port)) {
		cout << "Timeout while waiting for GCF/GRJ." << endl;
		return FALSE;
	}
		
	cout<< GetHostName() << " Rd from : " << gk_addr << " : " << gk_port << endl;
	grq_rpl.Decode( rdstrm );
	sock.Close();

	if(IsVerbose()) 
		cout << grq_rpl << endl;

	if( grq_rpl.GetTag() == H225_RasMessage::e_gatekeeperConfirm )
	{
		discoveryComplete = TRUE;
		//   grq_rpl.PrintOn( cout );
		return TRUE;
	}
	else 
		cout << "Can not discover Gatekeeper." << endl;
	return FALSE;
} 

BOOL RasClient::SendRRQ(PArgList &args)
{
	H225_RasMessage rrq_ras, rrq_rpl;
	rrq_ras.SetTag(H225_RasMessage::e_registrationRequest);
	H225_RegistrationRequest & rrq_obj = rrq_ras;

	rrq_obj.m_requestSeqNum.SetValue( seq_num++ );	
	rrq_obj.m_protocolIdentifier.SetValue( H225_ProtocolID);
	rrq_obj.m_discoveryComplete.SetValue( discoveryComplete );

	rrq_obj.m_callSignalAddress.SetSize(1);
	rrq_obj.m_callSignalAddress[0].SetTag(H225_TransportAddress::e_ipAddress);
	H225_TransportAddress_ipAddress & ipAddress = rrq_obj.m_callSignalAddress[0];
	ipAddress.m_ip[0]  = my_addr.Byte1();
	ipAddress.m_ip[1]  = my_addr.Byte2();
	ipAddress.m_ip[2]  = my_addr.Byte3();
	ipAddress.m_ip[3]  = my_addr.Byte4();
	ipAddress.m_port  = sig_port; 

	rrq_obj.m_rasAddress.SetSize(1);
	rrq_obj.m_rasAddress[0].SetTag(H225_TransportAddress::e_ipAddress);
	H225_TransportAddress_ipAddress & rasAddress = rrq_obj.m_rasAddress[0];
	rasAddress.m_ip[0]  = my_addr.Byte1();
	rasAddress.m_ip[1]  = my_addr.Byte2();
	rasAddress.m_ip[2]  = my_addr.Byte3();
	rasAddress.m_ip[3]  = my_addr.Byte4();
	rasAddress.m_port  = gk_port;  
    
	rrq_obj.IncludeOptionalField( rrq_obj.e_terminalAlias );    

	rrq_obj.m_terminalAlias.SetSize(1);
	rrq_obj.m_terminalAlias[0].SetTag(H225_AliasAddress::e_h323_ID);
	(PASN_BMPString &)rrq_obj.m_terminalAlias[0] = GetHostName();

	if (args.HasOption('u')) {
		PStringArray aliases = args.GetOptionString('u').Lines();
		int i, s=aliases.GetSize();

		rrq_obj.m_terminalType.IncludeOptionalField(H225_EndpointType::e_gateway);
		rrq_obj.m_terminalType.m_gateway.IncludeOptionalField(H225_GatewayInfo::e_protocol);
		rrq_obj.m_terminalType.m_gateway.m_protocol.SetSize(1);
		H225_SupportedProtocols &protocol=rrq_obj.m_terminalType.m_gateway.m_protocol[0];
		protocol.SetTag(H225_SupportedProtocols::e_voice);
		((H225_VoiceCaps &)protocol).m_supportedPrefixes.SetSize(s);

		rrq_obj.m_terminalAlias.SetSize(s+1);
    		for (i = 0; i < s; i++) {
			H323SetAliasAddress(aliases[i], rrq_obj.m_terminalAlias[i+1]);
			H323SetAliasAddress(aliases[i], ((H225_VoiceCaps &)protocol).m_supportedPrefixes[i].m_prefix);
		}
		// add for newer versions of OpenH323
		// rrq_obj.IncludeOptionalField(H225_RegistrationRequest::e_multipleCalls);
		// rrq_obj.m_multipleCalls.SetValue(TRUE);
	}


	PBYTEArray rdbuf(2048), wtbuf(2048);
	PPER_Stream rdstrm(rdbuf), wtstrm(wtbuf);

	rrq_ras.Encode ( wtstrm );
	wtstrm.CompleteEncoding();

        if(IsVerbose())
		cout << rrq_ras << endl;

	PUDPSocket sock( gk_port );
	if( !sock.Connect(gk_addr) )
	{
		cout << "Can not connect to the Gatekeeper." << endl;
		sock.Close();  
		return FALSE;
	}

	cout << GetHostName() << " Sent to " << gk_addr << ":" << gk_port << endl;
	sock.Write(wtstrm.GetPointer(), wtstrm.GetSize());

	sock.SetReadTimeout(GetReadTimeout());
	sock.ReadFrom( rdstrm.GetPointer(), rdstrm.GetSize(), gk_addr, gk_port);
	cout<< GetHostName() << " Rd from : " << gk_addr << " : " << gk_port << endl;
	rrq_rpl.Decode( rdstrm );
	sock.Close();  
    
	if( rrq_rpl.GetTag() == H225_RasMessage::e_registrationConfirm )
	{
		H225_RegistrationConfirm & rcf = rrq_rpl;
		endpointId = rcf.m_endpointIdentifier;
        	if(IsVerbose())
			cout << rrq_rpl << endl;
		return TRUE;
	}
	else if( rrq_rpl.GetTag() == H225_RasMessage::e_registrationReject )
	{ 
        	if(IsVerbose())
			cout << rrq_rpl << endl;
		return FALSE; 
	}
	else 
		cout << "Can not connect to the Gatekeeper" << endl;
	return FALSE;
}

BOOL RasClient::SendARQ(PString & destInfo, int bandWidth)
{
    H225_RasMessage arq_ras, arq_rpl;
    arq_ras.SetTag(H225_RasMessage::e_admissionRequest);
    H225_AdmissionRequest & arq_obj = arq_ras;

    arq_obj.m_requestSeqNum.SetValue( seq_num++ );	
    arq_obj.m_callType.SetTag(H225_CallType::e_pointToPoint);

    if( endpointId.IsEmpty() )
      (PASN_BMPString &)arq_obj.m_endpointIdentifier = GetHostName();
    else 
      (PASN_BMPString &)arq_obj.m_endpointIdentifier = endpointId;
    
    arq_obj.IncludeOptionalField( arq_obj.e_destinationInfo );    
    arq_obj.m_destinationInfo.SetSize(1);
    arq_obj.m_destinationInfo[0].SetTag(H225_AliasAddress::e_h323_ID);
    (PASN_BMPString &)arq_obj.m_destinationInfo[0] = destInfo;
    
    arq_obj.m_srcInfo.SetSize(1);
    arq_obj.m_srcInfo[0].SetTag(H225_AliasAddress::e_h323_ID);
    (PASN_BMPString &)arq_obj.m_srcInfo[0] = GetHostName();

    arq_obj.m_bandWidth = bandWidth;    
    arq_obj.m_callReferenceValue = 1;

    arq_obj.m_conferenceID = destInfo;
    arq_obj.m_activeMC = FALSE;
    arq_obj.m_answerCall = FALSE;

    PBYTEArray rdbuf(2048), wtbuf(2048);
    PPER_Stream rdstrm(rdbuf), wtstrm(wtbuf);

    arq_ras.Encode ( wtstrm );
    wtstrm.CompleteEncoding();
    // arq_ras.PrintOn( cout );

    PUDPSocket sock( gk_port );
    if( !sock.Connect(gk_addr) ) {
		cout << "Can not connect to the Gatekeeper." << endl;
		sock.Close();  
		return FALSE;
    }

    cout << GetHostName() <<" Sent to " << gk_addr << ":" << gk_port << endl;
    sock.Write(wtstrm.GetPointer(), wtstrm.GetSize());

	sock.SetReadTimeout(GetReadTimeout());
    sock.ReadFrom( rdstrm.GetPointer(), rdstrm.GetSize(), gk_addr, gk_port);
    cout<< GetHostName() << " Rd from : " << gk_addr << " : " << gk_port << endl;
    arq_rpl.Decode( rdstrm );
    sock.Close();  
    
    if( arq_rpl.GetTag() == H225_RasMessage::e_admissionConfirm )
	{
      //arq_rpl.PrintOn( cout );  
      return TRUE;
    }
    else if( arq_rpl.GetTag() == H225_RasMessage::e_admissionReject )
	{
      // arq_rpl.PrintOn( cout );
	}
    else 
		cout << "Can not connect to the Gatekeeper" << endl;    
    return FALSE;
}

BOOL RasClient::SendDRQ( PString & destInfo)
{
    H225_RasMessage drq_ras, drq_rpl;
    drq_ras.SetTag(H225_RasMessage::e_disengageRequest);
    H225_DisengageRequest & drq_obj = drq_ras;

    drq_obj.m_requestSeqNum.SetValue( seq_num++ );

    if( endpointId.IsEmpty() )
      (PASN_BMPString &)drq_obj.m_endpointIdentifier = GetHostName();
    else 
      (PASN_BMPString &)drq_obj.m_endpointIdentifier = endpointId;

    drq_obj.m_conferenceID = destInfo;
    drq_obj.m_callReferenceValue = 1;

    drq_obj.m_disengageReason.SetTag( H225_DisengageReason::e_normalDrop );
    
    // Send DRQ
    PBYTEArray rdbuf(2048), wtbuf(2048);
    PPER_Stream rdstrm(rdbuf), wtstrm(wtbuf);

    drq_ras.Encode ( wtstrm );
    wtstrm.CompleteEncoding();
    //    drq_ras.PrintOn( cout );

   PUDPSocket sock( gk_port );
   if( !sock.Connect(gk_addr) ) {
      cout << "Can not connect to the Gatekeeper." << endl;
      sock.Close();  
      return FALSE;
    }
   
    cout << GetHostName() << " Sent to " << gk_addr << ":" << gk_port << endl;
    sock.Write(wtstrm.GetPointer(), wtstrm.GetSize());

	sock.SetReadTimeout(GetReadTimeout());
    sock.ReadFrom( rdstrm.GetPointer(), rdstrm.GetSize(), gk_addr, gk_port);
    cout<< GetHostName() << " Rd from : " << gk_addr << " : " << gk_port << endl;
    drq_rpl.Decode( rdstrm );
    sock.Close();

    if( drq_rpl.GetTag() == H225_RasMessage::e_disengageConfirm )
	{
      // drq_rpl.PrintOn( cout );  
      return TRUE;
    }
    else if( drq_rpl.GetTag() == H225_RasMessage::e_disengageReject )
	{
      // drq_rpl.PrintOn( cout );
	}
    else 
      cout << "Can not connect to the Gatekeeper" << endl;

    return FALSE;
} 


BOOL RasClient::SendLRQ(PString & destInfo)
{
    H225_RasMessage lrq_ras, lrq_rpl;
    lrq_ras.SetTag(H225_RasMessage::e_locationRequest);
    H225_LocationRequest & lrq_obj = lrq_ras;

    lrq_obj.m_requestSeqNum.SetValue( seq_num++ );

	// endpointIdentifier is optional in this message
    if( ! endpointId.IsEmpty() ) {
		lrq_obj.IncludeOptionalField( lrq_obj.e_endpointIdentifier );
		(PASN_BMPString &)lrq_obj.m_endpointIdentifier = endpointId;	// ??
	};

	// set replyAddress (mandatory)
	lrq_obj.m_replyAddress.SetTag(H225_TransportAddress::e_ipAddress);
	H225_TransportAddress_ipAddress & ipAddress = lrq_obj.m_replyAddress;
	ipAddress.m_ip[0]  = my_addr.Byte1();
	ipAddress.m_ip[1]  = my_addr.Byte2();
	ipAddress.m_ip[2]  = my_addr.Byte3();
	ipAddress.m_ip[3]  = my_addr.Byte4();
	ipAddress.m_port  = gk_port;

	// this is the alias we are asking the GK for
    lrq_obj.m_destinationInfo.SetSize(1);
    H323SetAliasAddress(destInfo, lrq_obj.m_destinationInfo[0]);

    // Send LRQ
    PBYTEArray rdbuf(2048), wtbuf(2048);
    PPER_Stream rdstrm(rdbuf), wtstrm(wtbuf);

    lrq_ras.Encode ( wtstrm );
    wtstrm.CompleteEncoding();
    // lrq_ras.PrintOn( cout ); cout << endl;

   PUDPSocket sock( gk_port );
   if( !sock.Connect(gk_addr) ) {
      cout << "Can not connect to the Gatekeeper." << endl;
      sock.Close();  
      return FALSE;
    }
   
    cout << GetHostName() << " Sent to " << gk_addr << ":" << gk_port << endl;
    sock.Write(wtstrm.GetPointer(), wtstrm.GetSize());

	// wait for LCF / LRJ
	sock.SetReadTimeout(GetReadTimeout());
    sock.ReadFrom( rdstrm.GetPointer(), rdstrm.GetSize(), gk_addr, gk_port);
    cout<< GetHostName() << " Rd from : " << gk_addr << " : " << gk_port << endl;
    lrq_rpl.Decode( rdstrm );
    sock.Close();

    if( lrq_rpl.GetTag() == H225_RasMessage::e_locationConfirm )
	{
		H225_LocationConfirm & lcf_obj = lrq_rpl;
	    cout << "LocationConfirm: " << endl;
	    cout << destInfo << " = " << lcf_obj.m_rasAddress << endl;
      return TRUE;
    }
    else if( lrq_rpl.GetTag() == H225_RasMessage::e_locationReject )
	{
	    cout << "LocationReject" << endl;
	}
    else 
		cout << "Can not connect to the Gatekeeper" << endl;

    return FALSE;
} 

