// RasClient.h

#ifndef _rasclient_h_
#define _rasclient_h_

// #include <unistd.h>
#include <ptlib.h>
#include <ptlib/sockets.h>
#include <ptclib/asner.h>
#include "h225.h" 
#include "h323.h" 

const char H225_ProtocolID[]= "0.0.8.2250.0.2";

class RasClient : public PUDPSocket
{
  PCLASSINFO(RasClient, PUDPSocket)
    public:

  RasClient(const PString &GK_host_name) : 
	  m_verbose(FALSE),
	  m_gkid("OpenH323GK"),
	  m_rtimeout(PTimeInterval(5000))
	  {
		  seq_num  = 2;
		  
		  gk_port = 1719;
		  sig_port = 1720;
		  
		  GetHostAddress(my_addr);
		  PString gk( GK_host_name);
		  GetHostAddress( gk, gk_addr );
		  
		  discoveryComplete = FALSE;
	  }
  
  ~RasClient() {}

  const PTimeInterval& GetReadTimeout () const { return m_rtimeout; }
  void SetReadTimeout(const PTimeInterval &t) { m_rtimeout = t; }
  
  BOOL IsVerbose() const { return m_verbose; }
  void SetVerbose(BOOL v) { m_verbose = v; }

  const PString &GetGKID() const { return m_gkid; }
  /** Set the GK-ID in a GRQ that the GK should match.
   */
  void SetGKID(const PString &gkid) { m_gkid = gkid; }


  BOOL SendGRQ();
  BOOL SendRRQ(PArgList &);
  BOOL SendARQ(PString & destInfo, int bandWidth);
  BOOL SendDRQ(PString & destInfo);
  BOOL SendLRQ(PString & destInfo);
      
 protected:

  BOOL m_verbose;
  PString m_gkid;
  PTimeInterval m_rtimeout;

  int seq_num;
  WORD gk_port;
  WORD sig_port;
  Address gk_addr, my_addr;
    
  BOOL discoveryComplete;
  PString endpointId;
};
 
#endif
