// client.cxx
//
// Usage:  see option --help
//
// Changes:
//   - some minor output changes, exit status
//   - verbosity (only GRQ)
//   - gatekeeper ID
//   - read timeout (incomplete, only GRQ may recognize)

#include <ptlib.h>
#include <ptlib/sockets.h>
#include <ptclib/asner.h>
#include "h225.h" 
#include "h323.h" 
#include "RasClient.h"

class Client : public PProcess
{
  PCLASSINFO(Client, PProcess)
  public:
    void Main();
};
 
PCREATE_PROCESS(Client)
 
void Client::Main()
{
	PString gkName;
	RasClient *rasClient;
	BOOL getConfirm = TRUE;

	// Arg Processing
	PArgList & args = GetArguments();
	args.Parse("h-help.k-GK:g-GRQ.r-RRQ.a-ARQ:b:d-DRQ:l-LRQ:v-verbose.I-gkid:T-readtimeout:u:");

	if( args.HasOption('h') )
	{
		cout << "Program usage : \n\t -h --help" << endl <<
			"\t -v --verbose                            print out more things" << endl <<
			"\t -k(GateKeeperHostName) --GK             specify a GK host [localhost]" << endl <<
			"\t -I(GateKeeperID) --gkid                 the id of the GK for the GRQ [OpenH323GK]" << endl <<
			"\t -T(timeout) --readtimeout               the read timeout in s [5]" << endl <<
			"\t -g --GRQ                                send out GRQ request" << endl <<
			"\t -r --RRQ                                send out RRQ request " << endl <<
			"\t -u(alias)								register as gateway with alias (only valid with -r)" << endl <<
			"\t -a(DestHostName) [-b(bandwidth)] --ARQ  send out ARQ request " << endl <<
			"\t -d(DestHostName) --DRQ                  send out DRQ request " << endl <<
			"\t -l(alias) --LRQ                  		send out LRQ request " << endl;
		exit(0);
	}

	if (args.HasOption('k'))
		gkName = args.GetOptionString('k');
	else
		gkName = PIPSocket::GetHostName();

	rasClient = new RasClient( gkName );

	rasClient->SetVerbose(args.HasOption('v'));

	// Read-Timeout
	if( args.GetOptionString('T').GetLength() > 0 ) {
		const PString &s = args.GetOptionString('T');
		int i = s.AsInteger();
		if(i>0)
			rasClient->SetReadTimeout(PTimeInterval(i*1000));
	}

	// GK-ID
	if( args.GetOptionString('I').GetLength() > 0 )
		rasClient->SetGKID(args.GetOptionString('I'));

	// GRQ
	if ( args.HasOption('g'))
	{
		getConfirm = rasClient->SendGRQ();
	};

	// RRQ
	if ( args.HasOption('r'))
	{
		getConfirm = rasClient->SendRRQ(args);
	};


	// ARQ
	if ( args.HasOption('a'))
	{  
		PString destInfo;
	    int bandWidth = 640;	// default

		if( args.GetOptionString('a').GetLength() > 0 )
			destInfo = args.GetOptionString('a');
		else 
			destInfo = PIPSocket::GetHostName();

		if( args.HasOption('b'))
			bandWidth = atoi(args.GetOptionString('b'));
        
		getConfirm = rasClient->SendARQ( destInfo, bandWidth);
		if( !getConfirm )
		{
			delete rasClient;
			exit( 1 );
		}
	}

	// DRQ
	if ( args.HasOption('d'))
	{ 
		PString destInfo;

		if( args.GetOptionString('d').GetLength() > 0 )
			destInfo = args.GetOptionString('d');
		else 
			destInfo = PIPSocket::GetHostName();
    
		getConfirm = rasClient->SendDRQ( destInfo );
		if( !getConfirm )
		{
			delete rasClient;
			exit( 1 );
		}
	} 

	// LRQ
	if ( args.HasOption('l'))
	{ 
		PString destInfo;

		if( args.GetOptionString('l').GetLength() > 0 )
			destInfo = args.GetOptionString('l');
		else 
			destInfo = "";
    
		getConfirm = rasClient->SendLRQ( destInfo );
		if( !getConfirm )
		{
			delete rasClient;
			exit( 1 );
		}
	} 

	delete rasClient;

	// returns 0 if all tests were positive
	exit (getConfirm ? 0 : 1);
}

